unit Titlebar;
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Menus;

type TBtn = record
  L, T, W, H: Integer;
  Visible: Boolean;
  end;

type
  TTitlebar = class(TImage)
  private
     tmr: TTimer;
     MO: Integer;

    currentlychecking: boolean;
    CloseBtn: TBtn;
    MinBtn: TBtn;
    MaxBtn: TBtn;
    LeIcon: TBtn;
    MD: Byte;
    BMCB, BMMB, BMSB, BMRB: TBitmap;
    IsMax: Boolean;
    blnConcaved: Boolean;
    blnRaised: Boolean;
    blnResizable: Boolean;
    blnAutoRedraw: Boolean;

    blnAutoHide: Boolean;
    intAutoHideWidth: Integer;

    IsActive: Boolean;
    pmIcon: TPopupMenu;
    pmClose: TMenuItem;
    pmMin: TMenuItem;
    pmMax: TMenuItem;
    pmRestore: TMenuItem;
    alAlign: TAlign;

    strCaption: String;
    colr, decolr, frmcolr, btncolr: TColor;
    ourIcon: TIcon;
    blnUseIcon: Boolean;
    intBorderWidth: Integer;
    intWidth, intHeight, intLeft, intTop, intTBarWidth: Integer;

    procedure pmCloseOnClick(Sender: TObject);
    procedure pmMaxOnClick(Sender: TObject);
    procedure pmMinOnClick(Sender: TObject);
    procedure pmRestoreOnClick(Sender: TObject);

    procedure SetWidth(NewValue: Integer);
    procedure SetHeight(NewValue: Integer);

    procedure SetResizable(NewValue: Boolean);
    procedure SetRaised(NewValue: Boolean);
    procedure SetTBarWidth(NewValue: Integer);
    procedure SetConcaved(NewValue: Boolean);
    procedure SetCloseBtn(NewValue: Boolean);
    procedure SetMinBtn(NewValue: Boolean);
    procedure SetMaxBtn(NewValue: Boolean);
    procedure SetBorderWidth(NewValue: Integer);
    procedure SetUseIcon(NewValue: Boolean);
    procedure SetIcon(NewIcon: TIcon);
    procedure SetColor(NewColor: TColor);
    procedure SetDeColor(NewColor: TColor);
    procedure SetCaption(NewCaption: String);
    procedure SetAlign(NewAlign: TAlign);
    procedure SetFrmColr(NewValue: TColor);
    procedure SetBtnColr(NewValue: TColor);
    procedure SetAutoRedraw(NewValue: Boolean);
    procedure SetAutoHide(NewValue: Boolean);
    procedure SetAutoHideWidth(NewValue: Integer);

    procedure DeActivate(Sender: TObject);
    procedure Activate(Sender: TObject);
    procedure FreePopup;
    procedure RaisePopupMenu;

    procedure MakeBitmaps;
    procedure CalcAlign;
    procedure DrawSidewaysCanvas(X: Integer; Y: Integer);
    function GetTextColor(col: TColor): TColor;

    procedure CheckMouseOver;
    procedure OnTmer(Sender: TObject);
    { Private declarations }
  protected
    procedure MouseMove(Sender: TObject; Shift: TShiftState; X,
              Y: Integer);
    procedure MouseDown(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer); virtual;
    procedure MouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer); virtual;
    procedure DblClick(Sender: TObject); virtual;
    { Protected declarations }
  public
    procedure MakeTitleBar;
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
    { Public declarations }
  published
    { Published declarations }
      property FormColor: TColor read frmcolr write SetFrmColr;
      property CloseButton: Boolean read CloseBtn.Visible write SetCloseBtn;
      property MinButton: Boolean read MinBtn.Visible write SetMinBtn;
      property MaxButton: Boolean read MaxBtn.Visible write SetMaxBtn;
      property BorderWidth: Integer read intBorderWidth write SetBorderWidth;
      property UseIcon: Boolean read blnUseIcon write SetUseIcon;
      property Icon: TIcon read ourIcon write SetIcon;
      property Color: TColor read colr write SetColor default clBlue;
      property DeActiveColor: TColor read decolr write SetDeColor default clBlue;
      property Caption: String read strCaption write SetCaption;
      property Align: TAlign read alAlign write SetAlign default alTop;
      property Concaved: Boolean read blnConcaved write SetConcaved;
      property Raised: Boolean read blnRaised write SetRaised;
      property Width: Integer read intWidth write SetWidth;
      property Height: Integer read intHeight write Setheight;
      property TBarWidth: Integer read intTBarWidth write SetTBarWidth;
      property ButtonColor: TColor read btncolr write SetBtnColr;
      property Resizable: Boolean read blnResizable write SetResizable;
      property AutoRedraw: Boolean read blnAutoRedraw write SetAutoRedraw;
      property AutoHide: Boolean read blnAutoHide write SetAutoHide;
      property AutoHideWidth: Integer read intAutoHideWidth write SetAutoHideWidth;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Seths', [TTitlebar]);
end;

procedure TTitlebar.OnTmer(Sender: TObject);
var
  Po: TPoint;
begin

   GetCursorPos(Po);
   if ((Po.X > Parent.Left) and (Po.X < Parent.Left + Parent.Width) and
      (Po.Y > Parent.Top) and (Po.Y < Parent.Top + Parent.Height)
      ) or (IsActive = True) then MO := 10;

   if MO > 0 then MO := MO - 1;

   Application.ProcessMessages;

   if Align = alTop then begin
      if MO <> 0 then Parent.Top := Screen.Height - Parent.Height
                 else Parent.Top := Screen.Height - intAutoHideWidth;
   end else if Align = alLeft then begin
      if MO <> 0 then Parent.Left := Screen.Width - Parent.Width
                 else Parent.Left := Screen.Width - intAutoHideWidth;
   end else if Align = alRight then begin
      if MO <> 0 then Parent.Left := 0
                 else Parent.Left := -Parent.Width + intAutoHideWidth;
   end else if Align = alBottom then begin
      if MO <> 0 then Parent.Top := 0
                 else Parent.Top := -Parent.Height + intAutoHideWidth;
   end;

   if not blnAutoHide then begin
      MO := 0;
      tmr.Free;
      CurrentlyChecking := False;
   end;

end;

procedure TTitlebar.CheckMouseOver;
begin

   if CurrentlyChecking then exit;
   CurrentlyChecking := True;
   tmr := TTimer.Create(self);
   tmr.Interval := 100;
   tmr.OnTimer := OnTmer;
   tmr.Enabled := True;

end;

constructor TTitlebar.Create(Owner: TComponent);
begin

  inherited Create(Owner);
  BMCB := TBitmap.Create; //The buttons
  BMMB := TBitmap.Create;
  BMSB := TBitmap.Create;
  BMRB := TBitmap.Create;
  MakeBitmaps; // Draw the buttons
  IsMax := False; //Dont start off maximzed
  ourIcon := TIcon.Create; // our icon
  //Switch active color and deactive color events
  Application.OnDeactivate := DeActivate;
  Application.OnActivate := Activate;
  OnMouseDown := MouseDown;
  IsActive := True;
  //Titlebar eventa
  OnMouseMove := MouseMove;
  OnMouseUp := MouseUp;
  OnDblClick := DblClick;
  if Parent = nil then begin
     intTBarWidth := 30;
     colr := clActiveCaption;
     decolr := clInActiveCaption;
     frmcolr := clBtnFace;
     CloseBtn.Visible := True;
     MinBtn.Visible := True;
     MaxBtn.Visible := True;
     strCaption := 'Titlebar';
     intBorderWidth := 5;
     blnRaised := True;
     blnUseIcon := True;
     btncolr := clBtnFace;
     alAlign := Align;
     blnResizable := True;
     blnAutoRedraw := True;
  end else Align := Align;

end;

destructor TTitlebar.Destroy;
begin

   BMCB.Free;
   BMMB.Free;
   BMSB.Free;
   BMRB.Free;
   ourIcon.Free;
   inherited Destroy;

end;

procedure TTitleBar.FreePopup;
begin

   pmClose.Free;
   pmMax.Free;
   pmMin.Free;
   pmRestore.Free;
   pmIcon.Free;

end;

procedure TTitleBar.RaisePopupMenu;
var
  Po: TPoint;
begin

  MD := 40;
  pmIcon := TPopupMenu.Create( Self );
  pmClose := TMenuItem.Create( Self );
  pmClose.Caption := 'Close';
  pmClose.OnClick := pmCloseOnClick;
  pmMax := TMenuItem.Create( Self );
  pmMax.Caption := 'Maximize';
  pmMax.OnClick := pmMaxOnClick;
  pmMin := TMenuItem.Create( Self );
  pmMin.Caption := 'Minimize';
  pmMin.OnClick := pmMinOnClick;
  pmRestore := TMenuItem.Create( Self );
  pmRestore.Caption := 'Restore';
  pmRestore.OnClick := pmRestoreOnClick;
  pmIcon.Items.Add( pmRestore );
  pmIcon.Items.Add( pmMin );
  pmIcon.Items.Add( pmMax );
  pmIcon.Items.Add( pmClose );

   GetCursorPos(Po);
   if MinBtn.Visible then pmIcon.Items.Items[1].Enabled := True
                      else pmIcon.Items.Items[1].Enabled := False;
   if MaxBtn.Visible then begin
      pmIcon.Items.Items[2].Enabled := IsMax = False;
      pmIcon.Items.Items[0].Enabled := IsMax = True;
   end else begin
      pmIcon.Items.Items[2].Enabled := False;
      pmIcon.Items.Items[0].Enabled := False;
   end;
   pmIcon.Popup(Po.X, Po.Y);

end;

procedure TTitlebar.pmCloseOnClick(Sender: TObject);
begin

   FreePopup;
   Application.Terminate;

end;

procedure TTitlebar.pmMaxOnClick(Sender: TObject);
begin

   //Maximize
   FreePopup;
   ShowWindow(Parent.Handle, SW_MAXIMIZE);
   IsMax := True;
   MakeTitleBar;

end;

procedure TTitlebar.pmMinOnClick(Sender: TObject);
begin

   FreePopup;
   ShowWindow(Application.Handle, SW_MINIMIZE ); //Minimize
   MakeTitleBar;

end;

procedure TTitlebar.pmRestoreOnClick(Sender: TObject);
begin

   //Restore the form
   FreePopup;
   ShowWindow(Parent.Handle, SW_RESTORE);
   IsMax := False;
   MakeTitleBar;

end;

procedure TTitlebar.DeActivate(Sender: TObject);
begin

   IsActive := False;
   MakeTitleBar;

end;

procedure TTitlebar.Activate(Sender: TObject);
begin

   IsActive := True;
   MakeTitleBar;

end;

procedure TTitlebar.SetAutoHideWidth(NewValue: Integer);
begin

   intAutoHideWidth := NewValue;
   MakeTitleBar;

end;

procedure TTitlebar.SetAutoHide(NewValue: Boolean);
begin

   blnAutoHide := NewValue;
   if NewValue = True then MakeTitleBar;

end;

procedure TTitlebar.SetAutoRedraw(NewValue: Boolean);
begin

   blnAutoRedraw := NewValue;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitleBar.SetTBarWidth(NewValue: Integer);
Begin

   intTBarWidth := NewValue;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitleBar.SetBtnColr(NewValue: TColor);
begin

   btncolr := NewValue;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitleBar.SetWidth(NewValue: Integer);
begin
//
end;

procedure TTitleBar.SetHeight(NewValue: Integer);
begin
//
end;

procedure TTitlebar.SetResizable(NewValue: Boolean);
begin

   blnResizable := NewValue;

end;

procedure TTitlebar.SetFrmColr(NewValue: TColor);
begin

   frmcolr := NewValue;
   TForm(Parent).Color := NewValue;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitlebar.SetConcaved(NewValue: Boolean);
begin

   blnConcaved := NewValue;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitlebar.SetRaised(NewValue: Boolean);
begin

   blnRaised := NewValue;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitlebar.SetAlign(NewAlign: TAlign);
begin

   if ((NewAlign <> alNone) or (NewAlign <> alClient)) then alAlign := NewAlign;
   inherited Align := alClient;
   TForm(Parent).borderstyle := bsNone; //Makes sure its pretty
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitlebar.SetCloseBtn( NewValue: Boolean);
begin

   CloseBtn.Visible := NewValue;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitlebar.SetMinBtn( NewValue: Boolean);
begin

   MinBtn.Visible := NewValue;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitlebar.SetMaxBtn( NewValue: Boolean);
begin

   MaxBtn.Visible := NewValue;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitlebar.SetBorderWidth(NewValue: Integer);
begin

   intBorderWidth := NewValue;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitlebar.SetUseIcon(NewValue: Boolean);
begin

   blnUseIcon := NewValue;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitlebar.SetIcon(NewIcon: TIcon);
begin

   ourIcon.Assign( NewIcon );
   TForm(Parent).Icon := ourIcon;
   if blnAutoRedraw then MakeTitleBar;

end;


procedure TTitlebar.SetCaption(NewCaption: String);
begin

   strCaption := NewCaption;
   TForm(Parent).Caption := strCaption;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitlebar.SetColor(NewColor: TColor);
begin

   colr := NewColor;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitlebar.SetDeColor(NewColor: TColor);
begin

   decolr := NewColor;
   if blnAutoRedraw then MakeTitleBar;

end;

procedure TTitleBar.DblClick(Sender: TObject);
var
  Po: TPoint;
begin

GetCursorPos(Po);
if ((Po.X < Parent.Left + intLeft) or (Po.X > Parent.Left + intLeft + intWidth) or
    (Po.Y < Parent.Top + intTop) or (Po.Y > PArent.Top + intTop + intHeight)) then exit;
if MaxBtn.Visible then begin

   if IsMax = False then begin
      ShowWindow(Parent.Handle, SW_MAXIMIZE);
      IsMax := True;
   end else begin
      ShowWindow(Parent.Handle, SW_RESTORE);
      IsMax := False;
   end;
   MakeTitleBar;
   MD := 9; //Skip other button MouseDown events

end;

end;

procedure TTitlebar.MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  A: Byte;
begin if ((Cursor <> crDefault) and (Button = mbLeft)) then begin

   A := 0;
   if Cursor = crSizeWE then begin
      if X < Width div 2 then A := 1 else A := 2;
   end;
   if Cursor = crSizeNS then begin
      if Y < Height div 2 then A := 3 else A := 6;
   end;
   if Cursor = crSizeNWSE then begin
      if X < width div 2 then A := 4 else A := 8;
   end;
   if Cursor = crSizeNESW then begin
      if X < Width div 2 then A := 7 else A := 5;
   end;
   if A > 0 then begin
      ReleaseCapture;
      DefWindowProc( Parent.Handle, WM_SYSCOMMAND, SC_SIZE + A, 0);
      MakeTitleBar;
   end;

end else begin

  if MD <> 9 then begin
   if ((X > CloseBtn.L) and (X < CloseBtn.L + CloseBtn.W) and (Button = mbLeft) and
      (Y > CloseBtn.T) and (Y < CloseBtn.T + CloseBtn.H) and (CloseBtn.Visible = True)) then MD := 1

   else if ((X > MinBtn.L) and (X < MinBtn.L + MinBtn.W) and (Button = mbLeft) and
      (Y > MinBtn.T) and (Y < MinBtn.T + MinBtn.H) and (MinBtn.Visible = True)) then MD := 3

   else if ((X > MaxBtn.L) and (X < MaxBtn.L + MaxBtn.W) and (Button = mbLeft) and
      (Y > MaxBtn.T) and (Y < MaxBtn.T + MaxBtn.H) and (MaxBtn.Visible = True) and (IsMax = False)) then MD := 2

   else if ((X > MaxBtn.L) and (X < MaxBtn.L + MaxBtn.W) and (Button = mbLeft) and
      (Y > MaxBtn.T) and (Y < MaxBtn.T + MaxBtn.H) and (MaxBtn.Visible = True) and (IsMax = True)) then MD := 4

   else if ((X > LeIcon.L) and (X < LeIcon.L + LeIcon.W) and
            (Y > LeIcon.T) and (Y < LeIcon.T + LeIcon.H) and
            (LeIcon.Visible)) then RaisePopupMenu

   else MD := 0;
  end else MD := 0;

	If ((button = mbleft) and (MD = 0) and (IsMax = False) and
      (X > intLeft) and (X < intLeft + intWidth) and
      (Y > intTop) and (Y < intTop + intHeight)) then begin
		releasecapture;
      DefWindowProc( Parent.Handle, WM_syscommand, $F012, 0);
	end;
end;
   MakeTitleBar;

end;

procedure TTitlebar.MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  D: Integer;
begin

   if blnResizable = False then exit;
   D := intBorderWidth;

   if (((X < D) and (Y < D)) or
       ((Parent.Width - X < D) and (Parent.Height - Y < D))) then Cursor := crSizeNWSE else
   if ( ((Parent.Width - X < D)   and (Y < D)) or
        ((Parent.Height - Y < D) and (X < D))) then Cursor := crSizeNESW else
   if ((X < D) or (Parent.Width - X < D)) then Cursor := crSizeWE else
   if ((Y < D) or (Parent.Height - Y < D)) then Cursor := crSizeNS else
      Cursor := crDefault;

end;

procedure TTitlebar.MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin

   if ((X > CloseBtn.L) and (X < CloseBtn.L + CloseBtn.W) and
       (Y > CloseBtn.T) and (Y < CloseBtn.T + CloseBtn.H) and (MD = 1)) then

   else if ((X > MaxBtn.L) and (X < MaxBtn.L + MaxBtn.W) and
            (Y > MaxBtn.T) and (Y < MaxBtn.T + MaxBtn.H) and (MD = 2)) then

   else if ((X > MaxBtn.L) and (X < MaxBtn.L + MaxBtn.W) and
            (Y > MaxBtn.T) and (Y < MaxBtn.T + MaxBtn.H) and (MD = 4)) then

   else if ((X > MinBtn.L) and (X < MinBtn.L + MinBtn.W) and
            (Y > MinBtn.T) and (Y < MinBtn.T + MinBtn.H) and (MD = 3)) then

   else MD := 0;


// Draw the icon and caption

   case MD of
      1: Application.Terminate; // They hit the close button
      2: begin  //Maximize
            ShowWindow(Parent.Handle, SW_MAXIMIZE);
            IsMax := True;
         end;
      3: ShowWindow(Application.Handle, SW_MINIMIZE ); //Minimize
      4: begin            //Restore the form
            ShowWindow(Parent.Handle, SW_RESTORE);
            IsMax := False;
         end;
   end;
//  end;
   MD := 0;
   Cursor := crDefault;
   MakeTitleBar;

end;

procedure TTitleBar.MakeTitleBar;
var
  Loop, intStart, intFinish, intStep: Integer;
  LR, LG, LB, DR, DG, DB: Integer;
  TLR, TLG, TLB, TDR, TDG, TDB: Integer;
  I: Integer;
begin

   if Addr(Parent) = nil then exit;
   MakeBitmaps; //Partially excessive

   if blnAutoHide and (not CurrentlyChecking) then CheckMouseOver;
   intBorderWidth := intBorderWidth + 1;
   intwidth := intTBarWidth + 4;
   intHeight := intTBarWidth + 4;
   if Align = alTop then begin
      intLeft := intBorderWidth;
      intTop := intBorderWidth;
      intWidth := Parent.ClientWidth - (2 * intBorderWidth);
   end else
   if Align = alLeft then begin
      intLeft := intBorderWidth;
      intTop := intBorderWidth;
      intHeight := Parent.ClientHeight - (2 * intBorderWidth);
   end else
   if Align = alRight then begin
      intLeft := Parent.ClientWidth - intWidth - intBorderWidth;
      intTop := intBorderWidth;
      intHeight := Parent.ClientHeight - (2 * intBorderWidth);
   end else
   if Align = alBottom then begin
      intLeft := intBorderWidth;
      intTop := Parent.ClientHeight - intHeight - intBorderWidth;
      intWidth := Parent.ClientWidth - (2 * intBorderWidth);
   end;
   intBorderWidth := intBorderWidth - 1;

// Makes sure the canvas is large enough
   Picture.Bitmap.Width := Parent.Width;
   Picture.Bitmap.Height := Parent.Height;
   inherited Width := Parent.Width;
   inherited Height := Parent.Height;
//Draw the form
   Canvas.Pen.Color := frmcolr; Canvas.Brush.Color := frmcolr;
   Canvas.Rectangle(0,0,Parent.Width,Parent.Height);
if intBorderWidth > 0 then begin
   TLR := (Canvas.Pen.Color and $FF);
   TLG := (Canvas.Pen.Color and $FF00) div 255;
   TLB := (Canvas.Pen.Color and $FF0000) div 65535;
   TDR := TLR;   TDG := TLG;   TDB := TLB;
   LR := (255 - TLR) div intBorderWidth;
   LG := (255 - TLG) div intBorderWidth;
   LB := (255 - TLB) div intBorderWidth;
   DR := TLR div intBorderWidth;
   DG := TLG div intBorderWidth;
   DB := TLB div intBorderWidth;

   if blnConcaved then begin
      intFinish := intBorderWidth;
      intStart := 0;
      intStep := 1;
   end else begin
      intStart := intBorderWidth;
      intFinish := 0;
      intStep := -1;
   end;
   Loop := intStart;
   repeat

     if blnRaised then
        Picture.Bitmap.Canvas.Pen.Color := RGB( TLR, TLG, TLB ) else
        Picture.Bitmap.Canvas.Pen.Color := RGB( TDR, TDG, TDB );

     Picture.Bitmap.Canvas.MoveTo( Parent.Width - Loop, Loop);
     Picture.Bitmap.Canvas.LineTo( Loop, Loop);
     Picture.Bitmap.Canvas.LineTo( Loop, Parent.Height - Loop);

     if blnRaised then
       Picture.Bitmap.Canvas.Pen.Color := RGB( TDR, TDG, TDB ) else
       Picture.Bitmap.Canvas.Pen.Color := RGB( TLR, TLG, TLB );
     Canvas.Font.Color := Picture.Bitmap.Canvas.Pen.Color;

     Picture.Bitmap.Canvas.MoveTo( Loop, Parent.Height - Loop);
     Picture.Bitmap.Canvas.LineTo( Parent.Width - Loop, Parent.Height - Loop);
     Picture.Bitmap.Canvas.LineTo( Parent.Width - Loop, Loop);

     TLR := TLR + LR; TLG := TLG + LG; TLB := TLB + LB;
     TDR := TDR - DR; TDG := TDG - DG; TDB := TDB - DB;
     loop := loop + intStep;
  until Loop = intFinish + intStep;
end;

// Set the colors
   if IsActive then Canvas.Pen.Color := colr
               else Canvas.Pen.Color := decolr;
   Canvas.Brush.Color := Canvas.Pen.Color;
// Draw the main color
   Canvas.Rectangle(intLeft,intTop,intLeft + intWidth,intTop +intHeight);

   CalcAlign; // Figure out where things will be with current alignment

// Draw the Close button
   if CloseBtn.Visible then begin
      Canvas.CopyRect(
         Rect(CloseBtn.L, CloseBtn.T, CloseBtn.L + CloseBtn.W, CloseBtn.T + CloseBtn.H),
         BMCB.Canvas, Rect(0,0,20,20) );
   end;
// Draw the Maximize button
   if MaxBtn.Visible then begin
     if IsMax = False then begin
      Canvas.CopyRect(
         Rect(MaxBtn.L, MaxBtn.T, MaxBtn.L + MaxBtn.W, MaxBtn.T + MaxBtn.H),
         BMMB.Canvas, Rect(0,0,20,20) )
     end else begin
      Canvas.CopyRect(
         Rect(MaxBtn.L, MaxBtn.T, MaxBtn.L + MaxBtn.W, MaxBtn.T + MaxBtn.H),
         BMRB.Canvas, Rect(0,0,20,20) );
     end;
   end;
// Draw the Minimize button
   if MinBtn.Visible then begin
      Canvas.CopyRect(
         Rect(MinBtn.L, MinBtn.T, MinBtn.L + MinBtn.W, MinBtn.T + MinBtn.H),
         BMSB.Canvas, Rect(0,0,20,20) );
   end;
// Refresh everything
   Canvas.Refresh;
   Refresh;

end;

procedure TTitleBar.MakeBitmaps;
begin

   BMCB.Width := 20;
   BMCB.Height := 20;
   BMMB.Width := 20;
   BMMB.Height := 20;
   BMSB.Width := 20;
   BMSB.Height := 20;
   BMRB.Width := 20;
   BMRB.Height := 20;
   with BMCB.Canvas do begin

      Brush.Color := btncolr;
      Rectangle(0, 0, 20, 20);
      if MD = 1 then Pen.Color := clBlack else Pen.Color := clWhite;
      MoveTo(20, 0);
      LineTo(0, 0);
      LineTo(0, 19);
      if MD = 1 then Pen.Color := clWhite else Pen.Color := clBlack;
      LineTo(19, 19);
      LineTo(19, 0);

      MoveTo( 3, 2);
      LineTo( 16, 17);
      MoveTo( 3, 17);
      LineTo( 16, 2);

   end;

   with BMMB.Canvas do begin

      Brush.Color := btncolr;
      Rectangle(0, 0, 20, 20);
      if MD = 2 then Pen.Color := clBlack else Pen.Color := clWhite;
      MoveTo(20, 0);
      LineTo(0, 0);
      LineTo(0, 19);
      if MD = 2 then Pen.Color := clWhite else Pen.Color := clBlack;
      LineTo(19, 19);
      LineTo(19, 0);

      MoveTo( 3, 2);
      LineTo( 16, 2);
      MoveTo( 3, 3);
      LineTo( 16, 3);
      MoveTo( 3, 2);
      LineTo( 3, 17);
      LineTo( 16, 17);
      LineTo( 16, 2);

   end;

   with BMSB.Canvas do begin

      Brush.Color := btncolr;
      Rectangle(0, 0, 20, 20);
      if MD = 3 then Pen.Color := clBlack else Pen.Color := clWhite;
      MoveTo(20, 0);
      LineTo(0, 0);
      LineTo(0, 19);
      if MD = 3 then Pen.Color := clWhite else Pen.Color := clBlack;
      LineTo(19, 19);
      LineTo(19, 0);

      MoveTo( 3, 16);
      LineTo( 16, 16);
      MoveTo( 3, 17);
      LineTo( 16, 17);

   end;

   with BMRB.Canvas do begin

      Brush.Color := btncolr;
      Rectangle(0, 0, 20, 20);
      if MD = 4 then Pen.Color := clBlack else Pen.Color := clWhite;
      MoveTo(20, 0);
      LineTo(0, 0);
      LineTo(0, 19);
      if MD = 4 then Pen.Color := clWhite else Pen.Color := clBlack;
      LineTo(19, 19);
      LineTo(19, 0);

      MoveTo( 3, 2);
      LineTo( 12, 2);
      LineTo( 12, 12);
      LineTo( 3, 12);
      LineTo( 3, 3);

      MoveTo( 8, 8);
      LineTo( 17, 8);
      LineTo( 17, 17);
      LineTo( 8, 17);
      LineTo( 8, 8);

   end;

end;

procedure TTitlebar.CalcAlign;
var
   I: Integer;
   tempC: TColor;
begin

  if Canvas.Font <> nil then begin
     if IsActive then tempC := GetTextColor(colr) else
                      tempC := GetTextColor(decolr);
     Canvas.Font.Color := tempC;
  end;

  if Align = alNone then Align := alTop;

  if Canvas.Font <> nil then begin
     Canvas.Font.Height := intTBarWidth - 4;
  end;
  Canvas.Brush.Style := bsClear; 

  // Figure out Width for Icon
  I := intTBarWidth - 4;

  if ((Align = alTop) or (Align = alBottom)) then begin

   // Draw the icon and caption
   if blnUseIcon = True then begin
      LeIcon.Visible := True;
      LeIcon.L := intLeft + 4;
      LeIcon.T := intTop + 4;
      LeIcon.W := I;
      LeIcon.H := I;
      DrawIconEx( Canvas.Handle, LeIcon.L, LeIcon.T, ourIcon.Handle,
         I, I, 0, Canvas.Brush.Handle, DI_IMAGE);
      Canvas.Textout( intLeft + 4 + I, intTop + (intHeight div 2) - (Canvas.Textheight(strCaption) div 2), strCaption );
   end else begin
      LeIcon.Visible := False;
      Canvas.Textout( intLeft + 4, intTop + (intHeight div 2) - (Canvas.Textheight(strCaption) div 2), strCaption );
   end;

   // Figure out button rects
   CloseBtn.L := intLeft + intWidth - I - 4;
   CloseBtn.T := intTop + 4;
   CloseBtn.W := I;
   CloseBtn.H := I;

   if CloseBtn.Visible then begin
      MaxBtn.L := CloseBtn.L - I - 4;
      MaxBtn.T := CloseBtn.T;
      MaxBtn.W := CloseBtn.W;
      MaxBtn.H := CloseBtn.H;
   end else begin
      MaxBtn.L := CloseBtn.L;
      MaxBtn.T := CloseBtn.T;
      MaxBtn.W := CloseBtn.W;
      MaxBtn.H := CloseBtn.H;
   end;

   if MaxBtn.Visible then begin
      MinBtn.L := MaxBtn.L - I - 4;
      MinBtn.T := MaxBtn.T;
      MinBtn.W := MaxBtn.W;
      MinBtn.H := MaxBtn.H;
   end else begin
      MinBtn.L := MaxBtn.L;
      MinBtn.T := MaxBtn.T;
      MinBtn.W := MaxBtn.W;
      MinBtn.H := MaxBtn.H;
   end;

  end else begin
   // Draw the icon and caption
   if blnUseIcon = True then begin
      LeIcon.Visible := True;
      LeIcon.L := intLeft + 4;
      LeIcon.T := intTop + 4;
      LeIcon.W := I;
      LeIcon.H := I;
      DrawIconEx( Canvas.Handle, LeIcon.L, LeIcon.T, ourIcon.Handle,
         I, I, 0, Canvas.Brush.Handle, DI_IMAGE);
      DrawSidewaysCanvas(0, intTBarWidth + intBorderWidth + 4 );
   end else begin
      DrawSidewaysCanvas(0, intTBarWidth + intBorderWidth + 4);
   end;
   // Figure out button rects
   CloseBtn.L := intLeft + 4;
   CloseBtn.T := intTop + intHeight - I - 4;
   CloseBtn.W := I;
   CloseBtn.H := I;
   if CloseBtn.Visible then begin
      MaxBtn.L := CloseBtn.L;
      MaxBtn.T := CloseBtn.T - I - 4;
      MaxBtn.W := CloseBtn.W;
      MaxBtn.H := CloseBtn.H;
   end else begin
      MaxBtn.L := CloseBtn.L;
      MaxBtn.T := CloseBtn.T;
      MaxBtn.W := CloseBtn.W;
      MaxBtn.H := CloseBtn.H;
   end;
   if MaxBtn.Visible then begin
      MinBtn.L := MaxBtn.L;
      MinBtn.T := MaxBtn.T - I - 4;
      MinBtn.W := MaxBtn.W;
      MinBtn.H := MaxBtn.H;
   end else begin
      MinBtn.L := MaxBtn.L;
      MinBtn.T := MaxBtn.T;
      MinBtn.W := MaxBtn.W;
      MinBtn.H := MaxBtn.H;
   end;

  end;

end;

procedure TTitlebar.DrawSidewaysCanvas(X: Integer; Y: Integer);
var
   T, I: Integer;
begin

   strCaption := strCaption + ' ';
   T := Y;
   I := 1;
   repeat
      Canvas.TextOut( intLeft + (intWidth div 2) - (Canvas.TextWidth(strCaption[I]) div 2), T, strCaption[I] );
      I := I + 1;
      T := T + Canvas.TextHeight(strCaption[I]);
   until I = Length(strCaption) + 1;
   strCaption := Copy(strCaption, 1, Length(strCaption) - 1);

end;

function TTitlebar.GetTextColor(col: TColor): TColor;
var
  R,G,B: Byte;
begin

   B := (col and $FF0000) div 65535;
   G := (col and $FF00) div 255;
   R := (col and $FF);

   if R + 128 < 256 then R := R + 128 else R := R - 128;
   if G + 128 < 256 then G := G + 128 else G := G - 128;
   if B + 128 < 256 then B := B + 128 else B := B - 128;

   Result := RGB(R,G,B);
   if col = clBlack then Result := clGray;

end;

end.
